package org.owasp.csrf.http;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.ByteArrayOutputStream;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

public final class MutableHttpResponse extends HttpServletResponseWrapper
{    
    private HttpServletResponse response = null;
    
    private ByteArrayOutputStream output = null;
    
    private FilterOutputStream stream = null;
    
    private PrintWriter writer = null;
    
    private int contentLength = -1;
    
    public MutableHttpResponse(HttpServletResponse response)
    {
        super(response);
        
        this.response = response;
        
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
    }
    
    /**
     * DEVNOTE: The following methods override javax.servlet.ServletResponse methods
     */
    public void flushBuffer() throws IOException
    {
        output.flush();
        stream.flush();
        writer.flush();
    }
    
    public int getBufferSize()
    {
        return output.size();
    }
    
    public ServletOutputStream getOutputStream()
    {
        return stream;
    }
    
    public PrintWriter getWriter() throws IOException
    {
        return writer;
    }
    
    public boolean isCommitted()
    {
        return output.size() != 0;
    }
    
    public void reset() throws IllegalStateException
    {
        if(isCommitted())
        {
            throw new IllegalStateException("the response has already been committed");
        }
        
        /** Clear any buffers **/
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
        
        /** Reset the status code and headers **/
        this.response.reset();
    }
    
    public void resetBuffer() throws IllegalStateException
    {
        if(isCommitted())
        {
            throw new IllegalStateException("the response has already been committed");
        }
        
        /** Clear any buffers **/
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
    }
    
    public void setContentLength(int length)
    {
        this.contentLength = length;
        super.setContentLength(length);
    }
    
    public byte[] getContent() throws IOException
    {
        /**
         * FIXME: Why do we have to flush here? Why isnt flushBuffer() getting called ?
         */
        writer.flush();
        stream.flush();
        output.flush();
        return output.toByteArray();
    }
    
    public void setContent(String s) throws IOException
    {
        /** Clear any buffers **/
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
        
        output.write(s.getBytes());
    }
    
    public int getContentLength()
    {
        return contentLength;
    }
    
    public HttpServletResponse getResponse()
    {
        return response;
    }
    
    public FilterOutputStream getStream()
    {
        return stream;
    }
    
    public ByteArrayOutputStream getOutput()
    {
        return output;
    }
}
